/*  XMPS - X MPEG Player System
 *  Copyright (C) 1999 Damien Chavarria
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/**
 *
 * opendivx_codec.c
 *
 * OPENDIVX DLL Codec.
 * @Author Chavarria Damien.
 * 	   Copyright 1999-2001
 */

#include "opendivx_codec.h"

typedef struct {

  xmps_system_plugin_t   *our_system;
  unsigned int            our_stream_id;
  xmps_video_info_t      *video_info;
  void                   *our_out_buffer;
  unsigned int           *next_size;

} opendivx_data_t;

/*
 * get_video_codec_info : MANDATORY
 *
 * Used by scanplugins to get a pointer to the codec.
 *
 */

xmps_video_decoder_plugin_t *get_video_decoder_info(void)
{
  xmps_video_decoder_plugin_t *video_decoder;

  video_decoder = (xmps_video_decoder_plugin_t *) malloc(sizeof(xmps_video_decoder_plugin_t));

  video_decoder->name = "OPENDIVX";
  video_decoder->data = malloc(sizeof(opendivx_data_t));

  video_decoder->open       = opendivx_open;
  video_decoder->set        = opendivx_set;
  video_decoder->get        = opendivx_get;
  video_decoder->decompress = opendivx_decompress;
  video_decoder->close      = opendivx_close;

  
  ((opendivx_data_t *) video_decoder->data)->video_info = (xmps_video_info_t *) malloc(sizeof(xmps_video_info_t));
  ((opendivx_data_t *) video_decoder->data)->our_system = NULL;

  return video_decoder;
}

unsigned int opendivx_open(xmps_video_decoder_plugin_t *video_decoder)
{
  if(video_decoder == NULL) {
    return 0;
  }

  return 1;
}

void* opendivx_get(xmps_video_decoder_plugin_t *video_decoder, unsigned int flag, void *arg)
{
  opendivx_data_t *data;

  if(video_decoder == NULL) {
    return 0;
  }

  data = (opendivx_data_t *) video_decoder->data;
 
  if(data == NULL) {
    return 0;
  }

  switch(flag)
    {
    case XMPS_FLAG_XML_CONFIG:
      {
	GList *list = NULL;
  
	return list;
      }
      break;

    case XMPS_FLAG_VIDEO_INFO:
      {
	
	if(data->our_system == NULL)
	  return NULL;
	
	return (void *) data->video_info;
      }
      break;
    case XMPS_FLAG_VIDEO_FORMAT_LIST:
      {
	GList             *formats = NULL;

	if(data->our_system == NULL)
	  return NULL;

	data->video_info->format.type = XMPS_VIDEO_FORMAT_YUV12;
	data->video_info->format.bpp  = 12;
	
	formats = g_list_append(formats, &data->video_info->format);
  
	return (void *) formats;
      }
      break;
    default:
      break;
    }
  
  return NULL;
}

unsigned int opendivx_set(xmps_video_decoder_plugin_t *video_decoder, unsigned int flag, void *arg)
{
  opendivx_data_t *data;

  if(video_decoder == NULL) {

    XMPS_DEBUG("problem!");
    return 0;
  }

  data = (opendivx_data_t *) video_decoder->data;
 
  if(data == NULL) {
    
    XMPS_DEBUG("problem!");
    return 0;
  }

  switch(flag)
    {

    case XMPS_FLAG_SAVE_CONFIG:
      {
	return 1;
      }
      break;
      
    case XMPS_FLAG_XML_CONFIG:

      return 1;
      break;

    case XMPS_FLAG_INPUT:
      {
	xmps_data_t *data_stream;

	data_stream = (xmps_data_t *) arg;

	if(data_stream == NULL) {
	  return 0;
	}

	XMPS_DEBUG("trying to open data stream as OpenDivX!");
	
	if(data->our_system != NULL) {

	  XMPS_DEBUG("our system is not NULL");
	  data->our_system = NULL;
	}
	
	if(data_stream->plugin_type == XMPS_PLUGIN_SYSTEM) {
	  data->our_system = data_stream->plugin;
	}

	if(data->our_system == NULL) {
	  return 0;
	}

	if(data_stream->type == XMPS_DATA_VIDEO_COMP && data_stream->video_compression != NULL) {
	  
	  XMPS_DEBUG("open : stream is video");

	  if( strstr("DIVX", data_stream->video_compression) != NULL ||
	      strstr("DVX1", data_stream->video_compression) != NULL ||
	      strstr("dvx1", data_stream->video_compression) != NULL ||
	      strstr("divx", data_stream->video_compression) != NULL)
	    {
	      DEC_PARAM dec_param;
	      DEC_SET   dec_set;

	      XMPS_DEBUG("found 1 video stream with compression %s", 
			 data_stream->video_compression);

	      /*
	       * init the decore
	       */ 


	      dec_param.x_dim = data_stream->video_info.width;
	      dec_param.y_dim = data_stream->video_info.height;

	      dec_param.color_depth = 32;
	      decore(0x8001, DEC_OPT_INIT, &dec_param, NULL);

	      dec_set.postproc_level = 0;
	      decore(0x8001, DEC_OPT_SETPP, &dec_set, NULL);

	      /*
	       * get the video info
	       */ 

	      data->video_info->width  = data_stream->video_info.width;
              data->video_info->height = data_stream->video_info.height;

              data->video_info->format.type = XMPS_VIDEO_FORMAT_YUV12;
              data->video_info->format.bpp  = 12;

	      /*
	       * And go
	       */ 

	      return 1;
	    }
	  else {
	    
	    XMPS_DEBUG("compresion was %s", data_stream->video_compression);
	    data->our_system = NULL;
	    return 0;
	  }
	}
	else {
	  return 0;
	}
	
      }
      break;

    case XMPS_FLAG_DROPFRAME:
      {
	DEC_FRAME dec_frame;

	if(data->next_size == NULL) {
	  
	  data->next_size = malloc(sizeof(unsigned int));
	  *data->next_size = 0;
	}

	dec_frame.length      = *data->next_size;
	dec_frame.bitstream   = arg;
	dec_frame.bmp         = NULL;
	dec_frame.render_flag = 0;
	
	decore(0x8001, 0, &dec_frame, NULL);    
      
	return 1;
	break;
      }
    default:
      break;
    }

  return 0;
}

unsigned int opendivx_decompress(xmps_video_decoder_plugin_t *video_decoder, void *in_buffer, void *out_buffer, unsigned int size)
{
  opendivx_data_t *data;
  DEC_FRAME dec_frame;

  if(video_decoder == NULL) {

    XMPS_DEBUG("null_video_decoder!");
    return 0;
  }

  data = (opendivx_data_t *) video_decoder->data;
 
  if(data == NULL) {

    XMPS_DEBUG("null data");
    return 0;
  }

  if(data->our_system == NULL) {

    XMPS_DEBUG("decompress : system not ready");
    return 0;
  }

  dec_frame.length      = size;
  dec_frame.bitstream   = in_buffer;
  dec_frame.bmp         = out_buffer;
  dec_frame.render_flag = 1;
  
  decore(0x8001, 0, &dec_frame, NULL);

  data->next_size = data->our_system->get(data->our_system, XMPS_FLAG_VIDEO_BUFFER_SIZE, NULL);

  return 1;
}

unsigned int opendivx_close(xmps_video_decoder_plugin_t *video_decoder)
{
  opendivx_data_t *data;
  
  if(video_decoder == NULL) {
    return 0;
  }
  
  data = (opendivx_data_t *) video_decoder->data;
 
  if(data == NULL) {
    return 0;
  }

  XMPS_DEBUG("close");
  
  data->our_system = NULL;
  
  return 1;
}



